@@ -24,15 +24,12 @@
 #include <glade/glade.h>
 #include <libgnomeui/gnome-color-picker.h>
 #include <libgnomeui/gnome-file-entry.h>
+#include <libgnomeui/gnome-icon-entry.h>
 #include <string.h>
-#include "x-font-selector.h"
+#include "simple-x-font-selector.h"
+#include "terminal-widget.h"
 
-/* Bytes in a line of scrollback, rough estimate, including
- * data structure to hold the line. Based on reading
- * vt_newline in vt.c in libzvt. Each char in 80 columns
- * is a 32-bit int.
- */
-#define BYTES_PER_LINE (sizeof (void*) * 6 + (80.0 * 4))
+#define BYTES_PER_LINE (terminal_widget_get_estimated_bytes_per_scrollback_line ())
 
 typedef struct _TerminalColorScheme TerminalColorScheme;
 
@@ -74,6 +71,8 @@ static void       profile_editor_update_sensitivity          (GtkWidget       *e
                                                               TerminalProfile *profile);
 static void       profile_editor_update_visible_name         (GtkWidget       *editor,
                                                               TerminalProfile *profile);
+static void       profile_editor_update_icon                 (GtkWidget       *editor,
+                                                              TerminalProfile *profile);
 static void       profile_editor_update_cursor_blink         (GtkWidget       *editor,
                                                               TerminalProfile *profile);
 static void       profile_editor_update_default_show_menubar (GtkWidget       *editor,
@@ -127,6 +126,10 @@ static void       profile_editor_update_backspace_binding    (GtkWidget       *w
 static void       profile_editor_update_delete_binding       (GtkWidget       *widget,
                                                               TerminalProfile *profile);
 
+static void       profile_editor_update_use_theme_colors     (GtkWidget       *widget,
+                                                              TerminalProfile *profile);
+static void       profile_editor_update_use_system_font      (GtkWidget       *widget,
+                                                              TerminalProfile *profile);
 
 
 static void profile_forgotten (TerminalProfile     *profile,
@@ -206,6 +209,9 @@ profile_changed (TerminalProfile          *profile,
   if (mask & TERMINAL_SETTING_VISIBLE_NAME)
     profile_editor_update_visible_name (editor, profile);
 
+  if (mask & TERMINAL_SETTING_ICON)
+    profile_editor_update_icon (editor, profile);
+  
   if (mask & TERMINAL_SETTING_CURSOR_BLINK)
     profile_editor_update_cursor_blink (editor, profile);
 
@@ -284,6 +290,12 @@ profile_changed (TerminalProfile          *profile,
 
   if (mask & TERMINAL_SETTING_DELETE_BINDING)
     profile_editor_update_delete_binding (editor, profile);
+
+  if (mask & TERMINAL_SETTING_USE_THEME_COLORS)
+    profile_editor_update_use_theme_colors (editor, profile);
+    
+  if (mask & TERMINAL_SETTING_USE_SYSTEM_FONT)
+    profile_editor_update_use_system_font (editor, profile);
   
   profile_editor_update_sensitivity (editor, profile);
 }
@@ -305,6 +317,20 @@ visible_name_changed (GtkWidget       *entry,
   g_free (text);
 }
 
+static void
+icon_changed (GtkWidget       *icon_entry,
+              TerminalProfile *profile)
+{
+  char *filename;
+
+  filename = gnome_icon_entry_get_filename (GNOME_ICON_ENTRY (icon_entry));
+
+  /* NULL filename happens here to unset */
+  terminal_profile_set_icon_file (profile, filename);
+  
+  g_free (filename);
+}
+
 static void
 cursor_blink_toggled (GtkWidget       *checkbutton,
                       TerminalProfile *profile)
@@ -426,6 +452,15 @@ word_chars_changed (GtkWidget       *entry,
   g_free (text);
 }
 
+static void 
+font_changed (EggXFontSelector *selector, TerminalProfile *profile)
+{
+  gchar *name = egg_xfont_selector_get_font_name (selector);
+  if (name)
+    terminal_profile_set_x_font (profile, name);
+  g_free (name);
+}
+
 static void
 scrollbar_position_changed (GtkWidget       *option_menu,
                             TerminalProfile *profile)
@@ -558,66 +593,6 @@ palette_color_set (GtkWidget       *colorpicker,
                                       &color);
 }
 
-static void
-font_dialog_destroyed (GtkWidget *dialog,
-                       gpointer   data)
-{
-  GtkWidget *button;
-
-  button = data;
-
-  g_object_set_data (G_OBJECT (button), "font-selector", NULL);
-}
-
-static void
-x_font_clicked (GtkWidget       *button,
-                TerminalProfile *profile)
-{
-  /* Pop up dialog, get new font, terminal_profile_set_x_font */
-  GtkWidget *dialog;
-  GtkWidget *parent;
-  
-  dialog = g_object_get_data (G_OBJECT (button),
-                              "font-selector");
-
-  if (dialog)
-    {
-      gtk_window_present (GTK_WINDOW (dialog));
-      return;
-    }
-  
-  parent = gtk_widget_get_ancestor (button, GTK_TYPE_WINDOW);
-  g_assert (parent);
-
-#if 0
-  dialog = gnome_font_selector_new (_("Choose a font"));
-  
-  gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
-  gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (parent));
-#else
-  dialog = gtk_message_dialog_new (GTK_WINDOW (parent),
-                                   GTK_DIALOG_DESTROY_WITH_PARENT,
-                                   GTK_MESSAGE_INFO,
-                                   GTK_BUTTONS_CLOSE,
-                                   "Sadly, this button doesn't work. "
-                                   "To fix, repair x-font-selector.[hc] "
-                                   "then connect it up in %s (line %d, function %s); all "
-                                   "other parts of the font pref are already done.\n"
-                                   "http://bugzilla.gnome.org/show_bug.cgi?id=71744",
-                                   __FILE__, __LINE__,
-                                   G_GNUC_FUNCTION);
-  g_signal_connect (G_OBJECT (dialog), "response",
-                    G_CALLBACK (gtk_widget_destroy),
-                    NULL);
-#endif
-
-  g_object_set_data (G_OBJECT (button), "font-selector", dialog);
-  g_signal_connect (G_OBJECT (dialog), "destroy",
-                    G_CALLBACK (font_dialog_destroyed), button);
-
-  gtk_window_present (GTK_WINDOW (dialog));
-}
-
 static void
 solid_radio_toggled (GtkWidget       *radiobutton,
                      TerminalProfile *profile)
@@ -703,6 +678,29 @@ delete_binding_changed (GtkWidget       *option_menu,
   terminal_profile_set_delete_binding (profile, i);
 }
 
+static void
+reset_compat_defaults_clicked (GtkWidget       *button,
+                               TerminalProfile *profile)
+{
+  terminal_profile_reset_compat_defaults (profile);
+}
+
+static void
+use_theme_colors_toggled (GtkWidget       *checkbutton,
+                          TerminalProfile *profile)
+{
+  terminal_profile_set_use_theme_colors (profile,
+                                         gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton)));
+}
+
+static void
+use_system_font_toggled (GtkWidget       *checkbutton,
+                         TerminalProfile *profile)
+{
+  terminal_profile_set_use_system_font (profile,
+                                        gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton)));
+}
+
 /*
  * initialize widgets
  */
@@ -767,30 +765,13 @@ init_palette_scheme_menu (GtkWidget *option_menu)
                             menu);
 }
 
-static void
-edited_cb (GtkCellRenderer *renderer,
-	   gchar           *path_text,
-	   gchar           *new_text,
-	   GtkTreeModel    *model)
-{
-  GtkTreePath *path;
-  GtkTreeIter iter;
-
-  path = gtk_tree_path_new_from_string (path_text);
-  gtk_tree_model_get_iter (model, &iter, path);
-  gtk_list_store_set (GTK_LIST_STORE (model), &iter,
-		      0, new_text,
-		      -1);
-  gtk_tree_path_free (path);
-}
-
 void
 terminal_profile_edit (TerminalProfile *profile,
                        GtkWindow       *transient_parent)
 {
-  GtkWidget *editor;
+  GtkWidget *editor, *fontsel;
   GtkWindow *old_transient_parent;
-  
+
   editor = g_object_get_data (G_OBJECT (profile),
                               "editor-window");
 
@@ -800,6 +781,7 @@ terminal_profile_edit (TerminalProfile *profile,
       GtkWidget *w;
       double num1, num2;
       gint i;
+      GtkSizeGroup *size_group;
       
       if (g_file_test ("./"TERM_GLADE_FILE,
                        G_FILE_TEST_EXISTS))
@@ -855,6 +837,7 @@ terminal_profile_edit (TerminalProfile *profile,
       gtk_dialog_add_buttons (GTK_DIALOG (editor),
                               GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT,
                               NULL);
+      gtk_dialog_set_default_response (GTK_DIALOG (editor), GTK_RESPONSE_ACCEPT);
       /* End "we have no Glade 2" workarounds */
 
 
@@ -899,6 +882,12 @@ terminal_profile_edit (TerminalProfile *profile,
                         G_CALLBACK (visible_name_changed),
                         profile);
 
+      w = glade_xml_get_widget (xml, "profile-icon-entry");
+      profile_editor_update_icon (editor, profile);
+      g_signal_connect (G_OBJECT (w), "changed",
+                        G_CALLBACK (icon_changed),
+                        profile);
+      
       w = glade_xml_get_widget (xml, "blink-cursor-checkbutton");
       profile_editor_update_cursor_blink (editor, profile);
       g_signal_connect (G_OBJECT (w), "toggled",
@@ -925,6 +914,7 @@ terminal_profile_edit (TerminalProfile *profile,
 
       w = glade_xml_get_widget (xml, "color-scheme-optionmenu");
       init_color_scheme_menu (w);
+      profile_editor_update_color_scheme_menu (editor, profile);
       profile_editor_update_palette (editor, profile);
       g_signal_connect (G_OBJECT (w), "changed",
                         G_CALLBACK (color_scheme_changed),
@@ -1078,6 +1068,12 @@ terminal_profile_edit (TerminalProfile *profile,
       g_signal_connect (G_OBJECT (w), "changed",
                         G_CALLBACK (delete_binding_changed),
                         profile);
+
+      w = glade_xml_get_widget (xml, "use-theme-colors-checkbutton");
+      profile_editor_update_use_theme_colors (editor, profile);
+      g_signal_connect (G_OBJECT (w), "toggled",
+                        G_CALLBACK (use_theme_colors_toggled),
+                        profile);
       
       i = 0;
       while (i < TERMINAL_PALETTE_SIZE)
@@ -1102,19 +1098,34 @@ terminal_profile_edit (TerminalProfile *profile,
 
       profile_editor_update_palette (editor, profile);
 
-      w = glade_xml_get_widget (xml, "font-button");
-      /* keep button from expanding the dialog, and truncate on
-       * the left not on either side; would really prefer to
-       * center until running out of space, but oh well.
-       */
-      gtk_misc_set_alignment (GTK_MISC (gtk_bin_get_child (GTK_BIN (w))),
-                              0.0, 0.5);
-      gtk_widget_set_size_request (gtk_bin_get_child (GTK_BIN (w)),
-                                   0, -1);
+      w = glade_xml_get_widget (xml, "font-hbox");
+  
+      fontsel = egg_xfont_selector_new ();
+      g_object_set_data (G_OBJECT (editor), "font-selector", fontsel);
+ 
       profile_editor_update_x_font (editor, profile);
+      
+      g_signal_connect (fontsel, "changed",
+			G_CALLBACK (font_changed), profile);
+
+      size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+      gtk_size_group_add_widget (size_group,
+                                 EGG_XFONT_SELECTOR (fontsel)->family_label);
+      gtk_size_group_add_widget (size_group,
+                                 glade_xml_get_widget (xml,
+                                                       "profile-name-label"));
+      gtk_size_group_add_widget (size_group,
+                                 glade_xml_get_widget (xml,
+                                                       "profile-icon-label"));
+      g_object_unref (G_OBJECT (size_group));
+      
+      gtk_box_pack_start (GTK_BOX (w), GTK_WIDGET (fontsel), TRUE, TRUE, 0);
+      gtk_widget_show_all (w);
+      
+      w = glade_xml_get_widget (xml, "reset-compat-defaults-button");
       g_signal_connect (G_OBJECT (w), "clicked",
-                        G_CALLBACK (x_font_clicked),
-                        profile);
+			G_CALLBACK (reset_compat_defaults_clicked),
+			profile);
     }
   else
     {
@@ -1182,6 +1193,25 @@ profile_editor_update_sensitivity (GtkWidget       *editor,
       w = profile_editor_get_widget (editor, "scroll-background-checkbutton");
       gtk_widget_set_sensitive (w, FALSE);
     }
+
+  if (terminal_profile_get_use_theme_colors (profile))
+    {
+      set_insensitive (editor, "foreground-colorpicker", TRUE);
+      set_insensitive (editor, "background-colorpicker", TRUE);
+      set_insensitive (editor, "color-scheme-optionmenu", TRUE);
+    }
+  else
+    {      
+      set_insensitive (editor, "foreground-colorpicker",
+                       mask & TERMINAL_SETTING_FOREGROUND_COLOR);
+      
+      set_insensitive (editor, "background-colorpicker",
+                       mask & TERMINAL_SETTING_BACKGROUND_COLOR);
+      
+      set_insensitive (editor, "color-scheme-optionmenu",
+                       (mask & (TERMINAL_SETTING_BACKGROUND_COLOR |
+                                TERMINAL_SETTING_FOREGROUND_COLOR)));
+    }
   
 #if 0 /* uncomment once we've tested the sensitivity code */
   if (mask == last_mask)
@@ -1193,22 +1223,15 @@ profile_editor_update_sensitivity (GtkWidget       *editor,
   
   set_insensitive (editor, "profile-name-entry",
                    mask & TERMINAL_SETTING_VISIBLE_NAME);
+
+  set_insensitive (editor, "profile-icon-entry",
+                   mask & TERMINAL_SETTING_ICON);
   
   set_insensitive (editor, "blink-cursor-checkbutton",
                    mask & TERMINAL_SETTING_CURSOR_BLINK);
 
   set_insensitive (editor, "show-menubar-checkbutton",
                    mask & TERMINAL_SETTING_DEFAULT_SHOW_MENUBAR);
-  
-  set_insensitive (editor, "foreground-colorpicker",
-                   mask & TERMINAL_SETTING_FOREGROUND_COLOR);
-
-  set_insensitive (editor, "background-colorpicker",
-                   mask & TERMINAL_SETTING_BACKGROUND_COLOR);
-
-  set_insensitive (editor, "color-scheme-optionmenu",
-                   (mask & (TERMINAL_SETTING_BACKGROUND_COLOR |
-                            TERMINAL_SETTING_FOREGROUND_COLOR)));
 
   set_insensitive (editor, "title-entry",
                    mask & TERMINAL_SETTING_TITLE);
@@ -1255,7 +1278,7 @@ profile_editor_update_sensitivity (GtkWidget       *editor,
   set_insensitive (editor, "palette-optionmenu",
                    mask & TERMINAL_SETTING_PALETTE);
 
-  set_insensitive (editor, "font-button",
+  set_insensitive (editor, "font-hbox",
                    mask & TERMINAL_SETTING_X_FONT);
 
   set_insensitive (editor, "solid-radiobutton",
@@ -1272,6 +1295,9 @@ profile_editor_update_sensitivity (GtkWidget       *editor,
                    mask & TERMINAL_SETTING_BACKSPACE_BINDING);
   set_insensitive (editor, "delete-binding-optionmenu",
                    mask & TERMINAL_SETTING_DELETE_BINDING);
+
+  set_insensitive (editor, "use-theme-colors-checkbutton",
+                   mask & TERMINAL_SETTING_USE_THEME_COLORS);
   
   {
     int i;
@@ -1311,6 +1337,29 @@ profile_editor_update_visible_name (GtkWidget       *editor,
                              terminal_profile_get_visible_name (profile));
 }
 
+static void
+profile_editor_update_icon (GtkWidget       *editor,
+                            TerminalProfile *profile)
+{
+  GtkWidget *w;
+  char *current_filename;
+  const char *profile_filename;
+  
+  w = profile_editor_get_widget (editor, "profile-icon-entry");
+
+  current_filename = gnome_icon_entry_get_filename (GNOME_ICON_ENTRY (w));
+
+  profile_filename = terminal_profile_get_icon_file (profile);
+  
+  if (current_filename && profile_filename &&
+      strcmp (current_filename, profile_filename) == 0)
+    return;
+
+  g_free (current_filename);
+  
+  gnome_icon_entry_set_filename (GNOME_ICON_ENTRY (w), profile_filename);
+}
+
 static void
 profile_editor_update_cursor_blink (GtkWidget       *editor,
                                     TerminalProfile *profile)
@@ -1638,15 +1687,42 @@ static void
 profile_editor_update_x_font (GtkWidget       *editor,
                               TerminalProfile *profile)
 {
-  GtkWidget *w;
-  GtkWidget *child;
-  
-  w = profile_editor_get_widget (editor, "font-button");  
-  
-  child = gtk_bin_get_child (GTK_BIN (w));
+  GtkWidget *fontsel;
+  char *spacings[] = { "m", "c", NULL };
+  char *slants[] = { "r", "ot", NULL };
+  char *weights[] = { "medium", "regular", NULL };
+  gchar *name;
 
-  gtk_label_set_text (GTK_LABEL (child),
-                      terminal_profile_get_x_font (profile));
+  fontsel = g_object_get_data (G_OBJECT (editor), "font-selector");
+
+  /* If the current selector font is the same as the new font,
+   * don't do any work.
+   */
+  name = egg_xfont_selector_get_font_name (EGG_XFONT_SELECTOR (fontsel));
+  
+  if (name == NULL)
+    return;
+
+  if (strcmp (name, terminal_profile_get_x_font (profile)) == 0)
+    {
+      g_free (name);
+      return;
+    }
+  
+  g_free (name);
+  
+  egg_xfont_selector_set_font_name (EGG_XFONT_SELECTOR (fontsel),
+				    terminal_profile_get_x_font (profile));
+
+  egg_xfont_selector_set_filter (EGG_XFONT_SELECTOR (fontsel),
+				 EGG_XFONT_FILTER_BASE,
+				 EGG_XFONT_ALL,
+				 NULL,
+				 weights,
+				 slants,
+				 NULL,
+				 spacings,
+				 NULL);
 }
 
 static void
@@ -1735,6 +1811,32 @@ profile_editor_update_delete_binding (GtkWidget       *editor,
                                terminal_profile_get_delete_binding (profile));
 }
 
+static void
+profile_editor_update_use_theme_colors (GtkWidget       *editor,
+                                        TerminalProfile *profile)
+{
+  GtkWidget *w;
+
+  w = profile_editor_get_widget (editor, "use-theme-colors-checkbutton");
+  
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
+                                terminal_profile_get_use_theme_colors (profile));
+}
+
+static void
+profile_editor_update_use_system_font (GtkWidget       *editor,
+                                       TerminalProfile *profile)
+{
+  GtkWidget *w;
+
+#if 0
+  w = profile_editor_get_widget (editor, "use-system-font-checkbutton");
+  
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
+                                terminal_profile_get_use_system_font (profile));
+#endif
+}
+
 static GtkWidget*
 profile_editor_get_widget (GtkWidget  *editor,
                            const char *widget_name)
