@@ -133,10 +133,16 @@ gboolean terminal_widget_fork_command      (GtkWidget   *widget,
                                             const char  *path,
                                             char       **argv,
                                             char       **envp,
+                                            const char  *working_dir,
+                                            int         *child_pid,
                                             GError     **err);
 
 int terminal_widget_get_estimated_bytes_per_scrollback_line (void);
 
+void terminal_widget_write_data_to_child (GtkWidget  *widget,
+                                          const char *data,
+                                          int         len);
+
 G_END_DECLS
 
 #endif /* TERMINAL_WIDGET_H */
