@@ -1547,7 +1547,7 @@ new_window_callback (GtkWidget      *menuitem,
   terminal_app_new_terminal (terminal_app_get (),
                              terminal_screen_get_profile (window->priv->active_term),
                              NULL,
-                             FALSE, FALSE, NULL, NULL);
+                             FALSE, FALSE, NULL, NULL, NULL, NULL);
 }
 
 static void
@@ -1557,7 +1557,7 @@ new_tab_callback (GtkWidget      *menuitem,
   terminal_app_new_terminal (terminal_app_get (),
                              terminal_screen_get_profile (window->priv->active_term),
                              window,
-                             FALSE, FALSE, NULL, NULL);
+                             FALSE, FALSE, NULL, NULL, NULL, NULL);
 }
 
 static void
