@@ -24,9 +24,16 @@
 #include "terminal-window.h"
 #include "profile-editor.h"
 #include <gconf/gconf-client.h>
+#include <bonobo-activation/bonobo-activation-activate.h>
+#include <bonobo-activation/bonobo-activation-register.h>
+#include <bonobo/bonobo-exception.h>
+#include <bonobo/bonobo-listener.h>
 #include <libgnome/gnome-program.h>
+#include <libgnomeui/gnome-ui-init.h>
+#include <libgnomeui/gnome-client.h>
 #include <popt.h>
 #include <string.h>
+#include <gdk/gdkx.h>
 
 
 /* Settings storage works as follows:
@@ -68,6 +75,7 @@ struct _TerminalApp
 
 static GConfClient *conf = NULL;
 static TerminalApp *app = NULL;
+static gboolean terminal_factory_disabled = FALSE;
 
 static void         sync_profile_list            (gboolean use_this_list,
                                                   GSList  *this_list);
@@ -80,10 +88,9 @@ static void profile_list_notify   (GConfClient *client,
 static void refill_profile_treeview (GtkWidget *tree_view);
 
 static void terminal_app_get_clone_command   (TerminalApp *app,
+                                              int         *argc,
                                               char      ***argvp);
 
-static void spew_restart_command             (TerminalApp *app);
-
 static GtkWidget*       profile_optionmenu_new          (void);
 static void             profile_optionmenu_refill       (GtkWidget       *option_menu);
 static TerminalProfile* profile_optionmenu_get_selected (GtkWidget       *option_menu);
@@ -91,6 +98,13 @@ static void             profile_optionmenu_set_selected (GtkWidget       *option
                                                          TerminalProfile *profile);
 
 
+static gboolean save_yourself_callback (GnomeClient        *client,
+                                        gint                phase,
+                                        GnomeSaveStyle      save_style,
+                                        gboolean            shutdown,
+                                        GnomeInteractStyle  interact_style,
+                                        gboolean            fast,
+                                        void               *data);
 
 enum {
   OPTION_COMMAND = 1,
@@ -100,9 +114,12 @@ enum {
   OPTION_WINDOW_WITH_PROFILE_ID,
   OPTION_TAB_WITH_PROFILE_ID,
   OPTION_SHOW_MENUBAR,
-  OPTION_HIDE_MENUBAR,
+  OPTION_TOGGLE_MENUBAR,
   OPTION_GEOMETRY,
   OPTION_USE_FACTORY,
+  OPTION_DISABLE_FACTORY,
+  OPTION_TITLE,
+  OPTION_WORKING_DIRECTORY,
   OPTION_LAST
 };  
 
@@ -184,7 +201,7 @@ struct poptOption options[] = {
     '\0',
     POPT_ARG_NONE,
     NULL,
-    OPTION_HIDE_MENUBAR,
+    OPTION_TOGGLE_MENUBAR,
     N_("Turn off the menubar for the last-specified window; applies to only one window; can be specified once for each window you create from the command line."),
     NULL
   },
@@ -203,9 +220,36 @@ struct poptOption options[] = {
     POPT_ARG_NONE,
     NULL,
     OPTION_USE_FACTORY,
-    N_("Ask existing profterm process to open a new window, if possible. (Not yet implemented, http://bugzilla.gnome.org/show_bug.cgi?id=71442"),
+    N_("Ask existing profterm process to open a new window, if possible. Defaults to true"),
     NULL
   },
+  {
+    "disable-factory",
+    '\0',
+    POPT_ARG_NONE,
+    NULL,
+    OPTION_DISABLE_FACTORY,
+    N_("Do not register with the activation nameserver, do not re-use an active terminal"),
+    NULL
+  },
+  {
+    "title",
+    't',
+    POPT_ARG_STRING,
+    NULL,
+    OPTION_TITLE,
+    N_("Set the terminal's title"),
+    N_("TITLE")
+  },
+  {
+    "working-directory",
+    '\0',
+    POPT_ARG_STRING,
+    NULL,
+    OPTION_WORKING_DIRECTORY,
+    N_("Set the terminal's working directory"),
+    N_("DIRNAME")
+  },
   {
     NULL,
     '\0',
@@ -222,6 +266,8 @@ typedef struct
   char *profile;
   gboolean profile_is_id;
   char **exec_argv;
+  char *title;
+  char *working_dir;
 } InitialTab;
 
 typedef struct
@@ -246,6 +292,8 @@ initial_tab_new (const char *profile,
   it->profile = g_strdup (profile);
   it->profile_is_id = is_id;
   it->exec_argv = NULL;
+  it->title = NULL;
+  it->working_dir = NULL;
   
   return it;
 }
@@ -255,6 +303,8 @@ initial_tab_free (InitialTab *it)
 {
   g_free (it->profile);
   g_strfreev (it->exec_argv);
+  g_free (it->title);
+  g_free (it->working_dir);
   g_free (it);
 }
 
@@ -306,6 +356,207 @@ ensure_top_tab (GList **initial_windows_p)
   return it;
 }
 
+static void
+set_default_icon (const char *filename)
+{
+  GdkPixbuf *pixbuf;
+  GError *err;
+  GList *list;
+  
+  err = NULL;
+  pixbuf = gdk_pixbuf_new_from_file (filename, &err);
+
+  if (pixbuf == NULL)
+    {
+      g_printerr (_("Could not load icon \"%s\": %s\n"),
+                  filename, err->message);
+      g_error_free (err);
+
+      return;
+    }
+
+  list = NULL;
+  list = g_list_prepend (list, pixbuf);
+
+  gtk_window_set_default_icon_list (list);
+
+  g_list_free (list);
+  g_object_unref (G_OBJECT (pixbuf));
+}
+
+/*
+ *   Invoked remotely to instantiate a terminal with the
+ * given arguments.
+ */
+static void
+terminal_new_event (BonoboListener    *listener,
+		    const char        *event_name, 
+		    const CORBA_any   *any,
+		    CORBA_Environment *ev,
+		    gpointer           user_data)
+{
+  int i;
+  CORBA_sequence_CORBA_string *args;
+
+  if (strcmp (event_name, "new_terminal"))
+    {
+      g_warning ("Unknown event '%s' on terminal",
+		 event_name);
+      return;
+    }
+
+  printf ("Create new terminal with:\n");
+  args = any->_value;
+  for (i = 0; i < args->_length; i++)
+    printf ("  arg %d = '%s'\n", i, args->_buffer [i]);
+
+  g_return_if_fail (app != NULL);
+  terminal_app_new_terminal (app,
+                             terminal_profile_get_for_new_term (),
+                             NULL, FALSE, FALSE, NULL, NULL, NULL, NULL);
+}
+
+#define ACT_IID "OAFIID:GNOME_Terminal_Factory"
+
+static Bonobo_RegistrationResult
+terminal_register_as_factory (void)
+{
+  char *per_display_iid;
+  BonoboListener *listener;
+  Bonobo_RegistrationResult result;
+
+  listener = bonobo_listener_new (terminal_new_event, NULL);
+
+  per_display_iid = bonobo_activation_make_registration_id (
+    ACT_IID, DisplayString (gdk_display));
+
+  result = bonobo_activation_active_server_register (
+    per_display_iid, BONOBO_OBJREF (listener));
+
+  if (result != Bonobo_ACTIVATION_REG_SUCCESS)
+    bonobo_object_unref (BONOBO_OBJECT (listener));
+
+  g_free (per_display_iid);
+
+  return result;
+}
+
+static gboolean
+terminal_invoke_factory (int argc, char **argv)
+{
+  Bonobo_Listener listener;
+
+  switch (terminal_register_as_factory ())
+    {
+      case Bonobo_ACTIVATION_REG_SUCCESS:
+	/* we were the first terminal to register */
+	return FALSE;
+      case Bonobo_ACTIVATION_REG_NOT_LISTED:
+	g_printerr (_("It appears that you do not have gnome-terminal.server installed in a valid location. Factory mode disabled.\n"));
+        return FALSE;
+      case Bonobo_ACTIVATION_REG_ERROR:
+        g_printerr (_("Error registering terminal with the activation service; factory mode disabled.\n"));
+        return FALSE;
+      case Bonobo_ACTIVATION_REG_ALREADY_ACTIVE:
+        /* lets use it then */
+        break;
+    }
+
+  listener = bonobo_activation_activate_from_id (
+    ACT_IID, Bonobo_ACTIVATION_FLAG_EXISTING_ONLY, NULL, NULL);
+
+  if (listener != CORBA_OBJECT_NIL)
+    {
+      int i;
+      CORBA_any any;
+      CORBA_sequence_CORBA_string args;
+      CORBA_Environment ev;
+
+      CORBA_exception_init (&ev);
+
+      any._type = TC_CORBA_sequence_CORBA_string;
+      any._value = &args;
+
+      args._length = argc;
+      args._buffer = g_newa (CORBA_char *, args._length);
+      for (i = 0; i < args._length; i++)
+        args._buffer [i] = argv [i];
+
+      Bonobo_Listener_event (listener, "new_terminal", &any, &ev);
+      CORBA_Object_release (listener, &ev);
+      if (!BONOBO_EX (&ev))
+        return TRUE;
+
+      CORBA_exception_free (&ev);
+    }
+  else
+    g_printerr (_("Failed to retrieve terminal server from activation server\n"));
+
+  return FALSE;
+}
+
+static void
+strip_old_args (int *argcp, char **argv)
+{
+  const char *old_args[] = {
+    "--tclass", /* =TCLASS             Terminal class name */
+    "--font", /* =FONT                 Specifies font name*/
+    "--nologin", /*                    Do not start up shells as login shells */
+    "--login", /*                      Start up shells as login shells */
+    "--foreground", /* =COLOR          Foreground color */
+    "--background", /* =COLOR          Background color */
+    "--solid",  /*                     Solid background */
+    "--pixmap", /* =PIXMAP             Background pixmap */
+    "--bgscroll", /*                   Background pixmap scrolls */
+    "--bgnoscroll",  /*                Background pixmap does not scroll */
+    "--shaded", /*                     Shade background */
+    "--noshaded", /*                   Do not shade background */
+    "--transparent", /*                Transparent background */
+    "--utmp",  /*                      Update utmp entry */
+    "--noutmp",  /*                    Do not update utmp entry */
+    "--wtmp",  /*                      Update wtmp entry */
+    "--nowtmp",  /*                    Do not update wtmp entry */
+    "--lastlog",  /*                   Update lastlog entry */
+    "--nolastlog",  /*                 Do not update lastlog entry */
+    "--icon", /* =ICON                 Set the window icon */
+    "--termname", /* =TERMNAME         Set the TERM variable */
+    "--start-factory-server"  /*      Try to start a TerminalFactory */
+  };
+  int i;
+  int j;
+  
+  i = 1;
+  while (i < *argcp)
+    {
+      /* don't scan past "--" */
+      if (strcmp (argv[i], "--") == 0)
+        return;
+
+      j = 0;
+      while (j < (int) G_N_ELEMENTS (old_args))
+        {
+          if (strstr (argv[i], old_args[j]) == argv[i])
+            {
+              /* Get rid of this option */
+              g_printerr (_("Option \"%s\" is no longer supported in this version of gnome-terminal; you might want to create a profile with the desired setting, and use the new --window-with-profile option\n"),
+                          old_args[j]);
+
+              g_memmove (argv + i, argv + i + 1,
+                         sizeof (char*) * (*argcp - i - 1));
+              
+              *argcp -= 1;
+
+              goto next;
+            }
+          ++j;
+        }
+
+    next:
+      
+      ++i;
+    }
+}
+
 static GnomeModuleInfo module_info = {
   PACKAGE, VERSION, N_("Terminal"),
   NULL,
@@ -333,25 +584,16 @@ main (int argc, char **argv)
   char **post_execute_args;
   const char **args;
   char *default_geometry = NULL;
+  GnomeModuleRequirement reqs[] = {
+    { "1.102.0", LIBGNOMEUI_MODULE },
+    { NULL, NULL }
+  };
+  GnomeClient *sm_client;
   
   bindtextdomain (GETTEXT_PACKAGE, TERM_LOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
-  
-  gtk_init (&argc, &argv);
-  
-  gnome_program_init (PACKAGE, VERSION,
-                      &module_info,
-                      1, /* don't give it any args for now since option parsing doesn't go
-                          * through here
-                          */
-                      argv,
-                      GNOME_PARAM_APP_PREFIX, TERM_PREFIX,
-                      GNOME_PARAM_APP_SYSCONFDIR, TERM_SYSCONFDIR,
-                      GNOME_PARAM_APP_DATADIR, TERM_DATADIR,
-                      GNOME_PARAM_APP_LIBDIR, TERM_LIBDIR,
-                      NULL); 
-  
+
   /* pre-scan for -x and --execute options (code from old gnome-terminal) */
   post_execute_args = NULL;
   i = 1;
@@ -386,6 +628,26 @@ main (int argc, char **argv)
       ++i;
     }
   
+  gtk_init (&argc, &argv);
+  
+  module_info.requirements = reqs;
+  
+  gnome_program_init (PACKAGE, VERSION,
+                      &module_info,
+                      1, /* don't give it any args for now since option parsing doesn't go
+                          * through here
+                          */
+                      argv,
+                      GNOME_PARAM_APP_PREFIX, TERM_PREFIX,
+                      GNOME_PARAM_APP_SYSCONFDIR, TERM_SYSCONFDIR,
+                      GNOME_PARAM_APP_DATADIR, TERM_DATADIR,
+                      GNOME_PARAM_APP_LIBDIR, TERM_LIBDIR,
+                      NULL); 
+
+  set_default_icon (TERM_DATADIR"/pixmaps/gnome-terminal.png");  
+
+  strip_old_args (&argc, argv);
+  
   ctx = poptGetContext (PACKAGE, argc, (const char **) argv, options, 0);
 
   poptReadDefaultConfig (ctx, TRUE);
@@ -531,7 +793,7 @@ main (int argc, char **argv)
           }
           break;
           
-        case OPTION_HIDE_MENUBAR:
+        case OPTION_TOGGLE_MENUBAR:
           {
             InitialWindow *iw;
             
@@ -597,7 +859,61 @@ main (int argc, char **argv)
           break;
 
         case OPTION_USE_FACTORY:
-          /* FIXME */
+          /* this is a deprecated default anyway */
+          break;
+
+        case OPTION_DISABLE_FACTORY:
+          terminal_factory_disabled = TRUE;
+          break;
+
+        case OPTION_TITLE:
+          {
+            const char *title;
+            InitialTab *it;
+            
+            title = poptGetOptArg (ctx);
+
+            if (title == NULL)
+              {
+                g_printerr (_("Option --title requires an argument giving the title\n"));
+                return 1;
+              }
+
+            it = ensure_top_tab (&initial_windows);
+
+            if (it->title)
+              {
+                g_printerr (_("Two titles given for one tab\n"));
+                return 1;
+              }
+
+            it->title = g_strdup (title);
+          }
+          break;
+
+        case OPTION_WORKING_DIRECTORY:
+          {
+            const char *dir;
+            InitialTab *it;
+            
+            dir = poptGetOptArg (ctx);
+
+            if (dir == NULL)
+              {
+                g_printerr (_("Option --working-directory requires an argument giving the directory\n"));
+                return 1;
+              }
+
+            it = ensure_top_tab (&initial_windows);
+
+            if (it->working_dir)
+              {
+                g_printerr (_("Two working directories given for one tab\n"));
+                return 1;
+              }
+
+            it->working_dir = g_strdup (dir);
+          }
           break;
           
         case OPTION_LAST:          
@@ -606,7 +922,13 @@ main (int argc, char **argv)
           break;
         }
     }
-  
+
+  if (!terminal_factory_disabled)
+    {
+      if (terminal_invoke_factory (argc, argv))
+        return 0;
+    }
+
   if (next_opt != -1)
     {
       g_printerr (_("Error on option %s: %s.\nRun '%s --help' to see a full list of available command line options.\n"),
@@ -662,6 +984,12 @@ main (int argc, char **argv)
   terminal_profile_initialize (conf);
   sync_profile_list (FALSE, NULL);
 
+  sm_client = gnome_master_client ();
+  g_signal_connect (G_OBJECT (sm_client),
+                    "save_yourself",
+                    G_CALLBACK (save_yourself_callback),
+                    app);
+  
   tmp = initial_windows;
   while (tmp != NULL)
     {
@@ -706,7 +1034,9 @@ main (int argc, char **argv)
                                          iw->force_menubar_state,
                                          iw->menubar_state,
                                          it->exec_argv,
-                                         iw->geometry);
+                                         iw->geometry,
+                                         it->title,
+                                         it->working_dir);
 
               current_window = g_list_last (app->windows)->data;
             }
@@ -717,7 +1047,9 @@ main (int argc, char **argv)
                                          current_window,
                                          FALSE, FALSE,
                                          it->exec_argv,
-                                         NULL);
+                                         NULL,
+                                         it->title,
+                                         it->working_dir);
             }
           
           tmp2 = tmp2->next;
@@ -739,7 +1071,9 @@ main (int argc, char **argv)
                                  default_window_menubar_forced,
                                  default_window_menubar_state,
                                  NULL,
-                                 default_geometry);
+                                 default_geometry,
+                                 NULL,
+                                 NULL);
 
       g_free (default_geometry);
     }
@@ -753,8 +1087,6 @@ main (int argc, char **argv)
     }
   
   gtk_main ();
-
-  spew_restart_command (app);
   
   return 0;
 }
@@ -780,8 +1112,6 @@ terminal_window_destroyed (TerminalWindow *window,
 
   if (app->windows == NULL)
     gtk_main_quit ();
-
-  spew_restart_command (app);
 }
 
 void
@@ -791,7 +1121,9 @@ terminal_app_new_terminal (TerminalApp     *app,
                            gboolean         force_menubar_state,
                            gboolean         forced_menubar_state,
                            char           **override_command,
-                           const char      *geometry)
+                           const char      *geometry,
+                           const char      *title,
+                           const char      *working_dir)
 {
   TerminalScreen *screen;
 
@@ -818,6 +1150,12 @@ terminal_app_new_terminal (TerminalApp     *app,
   
   terminal_screen_set_profile (screen, profile);
 
+  if (title)
+    terminal_screen_set_dynamic_title (screen, title);
+
+  if (working_dir)
+    terminal_screen_set_working_dir (screen, working_dir);
+  
   if (override_command)    
     terminal_screen_set_override_command (screen, override_command);
   
@@ -838,8 +1176,6 @@ terminal_app_new_terminal (TerminalApp     *app,
   gtk_window_present (GTK_WINDOW (window));
 
   terminal_screen_launch_child (screen);
-
-  spew_restart_command (app);
 }
 
 static GList*
@@ -976,9 +1312,8 @@ sync_profile_list (gboolean use_this_list,
       terminal_profile_set_is_default (new_default, TRUE);
     }
 
-  g_assert (terminal_profile_get_count () > 0);
-  
-  spew_restart_command (app);
+  g_assert (terminal_profile_get_count () > 0);  
+
   if (app->new_profile_base_menu)
     profile_optionmenu_refill (app->new_profile_base_menu);
   if (app->manage_profiles_list)
@@ -1248,7 +1583,7 @@ terminal_app_new_profile (TerminalApp     *app,
       hbox = gtk_hbox_new (FALSE, PADDING);
 
       label = gtk_label_new_with_mnemonic (_("Profile _Name:"));
-      gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+      gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
       entry = gtk_entry_new ();
       app->new_profile_name_entry = entry;
 
@@ -1256,7 +1591,7 @@ terminal_app_new_profile (TerminalApp     *app,
       gtk_entry_set_activates_default (GTK_ENTRY (entry), TRUE);
       gtk_label_set_mnemonic_widget (GTK_LABEL (label), entry);
       
-      gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+      gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
       gtk_box_pack_end (GTK_BOX (hbox), entry, FALSE, FALSE, 0);
       
       gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);      
@@ -1264,7 +1599,7 @@ terminal_app_new_profile (TerminalApp     *app,
       hbox = gtk_hbox_new (FALSE, PADDING);
 
       label = gtk_label_new_with_mnemonic (_("_Base new profile on:"));
-      gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+      gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
       app->new_profile_base_menu = profile_optionmenu_new ();
       option_menu = app->new_profile_base_menu;
       if (default_base_profile)
@@ -1273,7 +1608,7 @@ terminal_app_new_profile (TerminalApp     *app,
       
       gtk_label_set_mnemonic_widget (GTK_LABEL (label), option_menu);
       
-      gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+      gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
       gtk_box_pack_end (GTK_BOX (hbox), option_menu, FALSE, FALSE, 0);
 
       gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
@@ -1695,6 +2030,34 @@ selection_changed_callback (GtkTreeSelection *selection,
                             count > 0);
 }
 
+static void
+profile_activated_callback (GtkTreeView       *tree_view,
+                            GtkTreePath       *path,
+                            GtkTreeViewColumn *column,
+                            TerminalApp       *app)
+{
+  TerminalProfile *profile;
+  GtkTreeIter iter;
+  GtkTreeModel *model;
+
+  model = gtk_tree_view_get_model (tree_view);
+
+  if (!gtk_tree_model_get_iter (model, &iter, path))
+    return;
+  
+  profile = NULL;
+  gtk_tree_model_get (model,
+                      &iter,
+                      COLUMN_PROFILE_OBJECT,
+                      &profile,
+                      -1);
+
+  if (profile)
+    terminal_app_edit_profile (app,
+                               profile,
+                               GTK_WINDOW (app->manage_profiles_dialog));
+}
+
 void
 terminal_app_manage_profiles (TerminalApp     *app,
                               GtkWindow       *transient_parent)
@@ -1773,6 +2136,11 @@ terminal_app_manage_profiles (TerminalApp     *app,
       gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
 
       app->manage_profiles_list = create_profile_list ();
+
+      g_signal_connect (G_OBJECT (app->manage_profiles_list),
+                        "row_activated",
+                        G_CALLBACK (profile_activated_callback),
+                        app);
       
       sw = gtk_scrolled_window_new (NULL, NULL);
       gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
@@ -1973,6 +2341,7 @@ profile_optionmenu_set_selected (GtkWidget       *option_menu,
 
 static void
 terminal_app_get_clone_command (TerminalApp *app,
+                                int         *argcp,
                                 char      ***argvp)
 {
   int n_windows;
@@ -1996,8 +2365,10 @@ terminal_app_get_clone_command (TerminalApp *app,
       
       tmp = tmp->next;
     }
-
+  
   argc = 1; /* argv[0] */
+
+  argc += 1; /* --use-factory or --disable-factory */
   
   argc += n_windows; /* one --with-window-profile-internal-id per window,
                       * for the first tab in that window
@@ -2010,12 +2381,22 @@ terminal_app_get_clone_command (TerminalApp *app,
                                */
 
   argc += n_tabs * 2; /* one "--command foo" per tab */
+
+  argc += n_tabs * 2; /* one "--title foo" per tab */
+
+  argc += n_tabs * 2; /* one "--working-directory foo" per tab */
   
   argv = g_new0 (char*, argc + 1);
 
   i = 0;
   argv[i] = g_strdup (EXECUTABLE_NAME);
   ++i;
+
+  if (terminal_factory_disabled)
+    argv[i] = g_strdup ("--disable-factory");
+  else
+    argv[i] = g_strdup ("--use-factory");
+  ++i;
   
   tmp = app->windows;
   while (tmp != NULL)
@@ -2032,6 +2413,7 @@ terminal_app_get_clone_command (TerminalApp *app,
           TerminalScreen *screen = tmp2->data;
           const char *profile_id;
           const char **override_command;
+          const char *title;
           
           profile_id = terminal_profile_get_name (terminal_screen_get_profile (screen));
           
@@ -2062,11 +2444,25 @@ terminal_app_get_clone_command (TerminalApp *app,
               ++i;
               
               flattened = g_strjoinv (" ", (char**) override_command);
-              argv[i] = g_shell_quote (flattened);
+              argv[i] = flattened;
               ++i;
 
               g_free (flattened);
             }
+
+          title = terminal_screen_get_dynamic_title (screen);
+          if (title)
+            {
+              argv[i] = g_strdup ("--title");
+              ++i;
+              argv[i] = g_strdup (title);
+              ++i;
+            }
+
+          argv[i] = g_strdup ("--working-directory");
+          ++i;
+          argv[i] = g_strdup (terminal_screen_get_working_dir (screen));
+          ++i;
           
           tmp2 = tmp2->next;
         }
@@ -2077,21 +2473,42 @@ terminal_app_get_clone_command (TerminalApp *app,
     }
 
   *argvp = argv;
+  *argcp = i;
 }
 
-static void
-spew_restart_command (TerminalApp *app)
+static gboolean
+save_yourself_callback (GnomeClient        *client,
+                        gint                phase,
+                        GnomeSaveStyle      save_style,
+                        gboolean            shutdown,
+                        GnomeInteractStyle  interact_style,
+                        gboolean            fast,
+                        void               *data)
 {
-  char **argv;
+  char **clone_command;
+  TerminalApp *app;
+  int argc;
   int i;
   
-  terminal_app_get_clone_command (app, &argv);
+  app = data;
+  
+  terminal_app_get_clone_command (app, &argc, &clone_command);
 
+  /* GnomeClient builds the clone command from the restart command */
+  gnome_client_set_restart_command (client, argc, clone_command);
+
+  /* Debug spew */
+  g_print ("Saving session: ");
   i = 0;
-  while (argv[i])
+  while (clone_command[i])
     {
-      g_print ("%s ", argv[i]);
+      g_print ("%s ", clone_command[i]);
       ++i;
     }
   g_print ("\n");
+
+  g_strfreev (clone_command);
+  
+  /* success */
+  return TRUE;
 }
